#-----------------------------------------------------------------------
# Teste de aleatorização.

# Uma senhora toma chá.
#
# A senhora (Muriel Bristol) declarou saber distringuir entre bedidas
# segundo a ordem com que o chá ou leite foram colocados na xícara.  O
# experimento consistiu em servir 8 xícaras da bebida, 4 em cada order
# (chá/leite e leite/chá).  No final a senhora deveria indicar quais as
# xícaras que estão em cada um dos dois grupos.  DETALHE: ela sabe
# distinguir mas não sabe classificar.

# Quantidade de maneiras de gerar dois grupos de 4 xícaras usando 8.
n_poss <- choose(8, 4)
n_poss

# De 4 xícaras selecionadas para um grupo, X representa o número de
# acertos.
n_acertar <- sapply(4:0,
                    FUN = function(i) {
                        # (formas de acertar x em 4) * (formas de errar x em 4).
                        choose(4, i) * choose(4, 4 - i)
                    })
n_acertar

# Pr(Acerto total) = 1/70 < 5%.
cumsum(n_acertar)/n_poss

# Comprovando por simulação.
v <- rep(0:1, each = 4)
mean(replicate(n = 100000,
               expr = all(sample(v) == v)))

# Essa forma de fazer a conta é mais realista mas dá na mesma.
mean(replicate(n = 100000,
               expr = all(sample(v) == sample(v))))

#-----------------------------------------------------------------------
# Exemplo com teste para a diferença de médias.

# Comprimentos de crânios de cães pré históricos.
m <- c(120, 107, 110, 116, 114, 111, 113, 117, 114, 112)
f <- c(110, 111, 107, 108, 110, 105, 107, 106, 111, 111)

plot(ecdf(m), xlim = range(c(m, f)), col = "cyan")
lines(ecdf(f), col = "magenta")
rug(m, col = "cyan")
rug(f, col = "magenta")

# Diferença de média.
d <- mean(m) - mean(f)
d

# Aplicando um teste t.
t.test(x = m, y = f, var.equal = TRUE)

# Todos as combinações possíveis.
choose(n = 20, k = 10)

#--------------------------------------------
# Com todas as combinações possíveis (exaustão).

# Para construir todas.
k <- combn(x = 1:20, m = 10)
dim(k)

# Vetor com os valores dos dois grupos.
mf <- c(m, f)

g <- integer(20)
D <- apply(k,
           MARGIN = 2,
           FUN = function(i) {
               g[i] <- 1L
               -diff(tapply(mf, g, FUN = mean))
           })

hist(D, col = "gray50")
rug(D)
abline(v = d, col = 2)

plot(ecdf(D), cex = 0)
rug(D)
abline(v = d, col = 2)

# P-valor do teste.
2 * sum(D >= d)/length(D)

#--------------------------------------------
# Com simulação (não necessáriamente exaustivo).

# Variáveis que indentifica os grupos.
g <- rep(1:2, each = 10)

# Apenas para conferir.
cbind(g, mf)

# Replicando a diferença para grupos formados por aleatorização.
D <- replicate(9999, {
    gg <- sample(g)
    -diff(tapply(mf, gg, FUN = mean))
})

# Tem que juntar a estatística observada com as simuladas.
D <- c(D, d)

hist(D, col = "gray50")
rug(D)
abline(v = d, col = 2)

plot(ecdf(D), cex = 0)
rug(D)
abline(v = d, col = 2)

# P-valor do teste.
2 * sum(D >= d)/length(D)

#-----------------------------------------------------------------------
# Teste para 3 grupos.

# Valores observados (y) e grupos (g).
y <- c(7, 8, 11, 12, 14, 19, 21, 122, 0, 2, 5, 9)
g <- gl(3, 4)

cbind(y, g)
plot(y ~ as.numeric(g))

# Valor observado da estatística F.
fobs <- anova(lm(y ~ g))[1, "F value"]

fsim <- replicate(9999, {
    gg <- sample(g)
    anova(lm(y ~ gg))[1, "F value"]
})

# Junta a estatística observada com as obtidas por simulação.
fsim <- c(fsim, fobs)

# P-valor do teste.
sum(fsim >= fobs)/length(fsim)

# (maneiras de agrupar 4 em 12) * (maneiras de agrupar 4 em 8).
choose(12, 4) * choose(8, 4)

#-----------------------------------------------------------------------
# Teste de aleatorização para a correlação.

# N = 5 para um par de medidas.
x <- c(4, 8, 2, 10, 9)
y <- c(3, 5, 1, 7, 8)

cbind(x, y)

# r0 <- cor(x, y)
r0 <- cor(x, y, method = "spearman")
r0

library(gtools)

# Todas as permutações possiveis = 5! = 120 do vetor x.
X <- permutations(n = length(x), r = length(x), v = x)
str(X)
head(X)

# As 120 correlações obtidas para cada arranjo.
r <- apply(X, MARGIN = 1, FUN = cor, y = y, method = "spearman")

# P-valor do teste.
sum(r >= r0)/length(r)

#-----------------------------------------------------------------------
# Exemplo da matriz de distância geográfica e similaridade entre
# espécies.

# s <- scan()
# dput(s)
cont <- c("EA", "Afr", "Mada", "Ori", "Aus", "NZ", "SouAfr", "NorAm")
s <- c(30, 14, 23, 30, -4, 2, -9, 50, 50, 40, 4, 9, -6, 54, 50, 11, 14,
       5, 61, 3, -16, -16, 15, 11, 3, 14, -6, 36)/100
sim <- matrix(NA,
              ncol = length(cont),
              nrow = length(cont),
              dimnames = list(cont, cont))
sim[lower.tri(sim)] <- s

# Coeficientes de associação entre espécies de tesourinha.
sim

# j <- scan()
# dput(j)
j <- c(1, 2, 1, 2, 3, 2, 1, 1, 2, 3, 4, 3, 2, 3, 4, 5, 4, 3, 1, 2, 3, 2,
       1, 4, 3, 5, 4, 1)
jum <- sim
jum[lower.tri(jum)] <- j

# Número de saltos entre continentes conforme movimento das placas.
jum

# Correlação entre os valores (fere a suposição de independência).
k <- cor(s, j)

# Empilha os valores de similaridade.
simd <- do.call(expand.grid, dimnames(sim))
simd$s <- c(sim)
# simd <- na.omit(simd)
simd

K <- replicate(499, {
    # Troca os nomes de linhas e colunas.
    i <- sample(cont)
    colnames(jum) <- i
    rownames(jum) <- i
    # Empilha os valores de pulos.
    jumi <- do.call(expand.grid, dimnames(jum))
    jumi$j <- c(jum)
    # jumi <- na.omit(jumi)
    # jumi
    # Junta as duas tabelas.
    da <- na.omit(merge(jumi, simd))
    cor(da$j, da$s)
})

head(K)
tail(K)

K <- c(k, K)

hist(K)
abline(v = k, col = 2)

plot(density(K))
rug(K)
abline(v = k, col = 2)

plot(ecdf(K))
rug(K)
abline(v = k, col = 2)

# Mantel Matrix Randomization Test.
RSiteSearch("mantel randomization")

jumd <- as.dist(jum)
simd <- as.dist(sim)
class(jumd)
jumd

library(ade4)

# Passar valores negativos porque a hipótese H_a é "greater".
mr <- mantel.randtest(-jumd, simd, nrepet = 999)
mr

plot(mr)

#-----------------------------------------------------------------------
# Índice de Moran para medir dependência espacial.

# http://gis.stackexchange.com/questions/161887/significance-test-for-morans-i-using-monte-carlo-simulation

browseURL("https://en.wikipedia.org/wiki/Moran's_I")

# Moran's I
# https://en.wikipedia.org/wiki/Moran's_I

# Coordenadas dos eventos.
x <- 1:8
y <- 1:8

# Matriz de pesos.
ind <- expand.grid(i = 1:length(x), j = 1:length(y))

f <- function(i, j) {
    u <- min(3, sum(abs(ind[i, ] - ind[j, ])))
    c(0, 1, sqrt(1/2), 0)[u + 1]
}

w <- matrix(0, nrow(ind), nrow(ind))
for (i in 1:nrow(ind)) {
    for (j in 1:nrow(ind)) {
        w[i, j] <- f(i, j)
    }
}
w <- w/sum(w)

image(w)

# Índice de Moran.
moran <- function(x, weights) {
    n <- length(x)
    z <- as.vector((x - mean(x))/sd(x))
    as.vector(z %*% weights %*% (z * sqrt(n/(n - 1))))
}

# Teste de permutação com gráfico.
ppt <- function(z, w, N = 10000, ...) {
    stat <- moran(z, w)
    sim <- replicate(N, moran(sample(z, length(z)), w))
    p.value <- mean((all <- c(stat, sim)) >= stat)
    hist(sim, sub = paste("p =", round(p.value, 4)), xlim = range(all),
         ...)
    abline(v = stat, col = "#903030", lty = 3, lwd = 2)
    return(p.value)
}

# Observações simuladas.
set.seed(17)
par(mfrow = c(2, 3))

# Indução de autocorrelação por meio de uma tendência.
z <- matrix(rexp(length(x) * length(y),
                 outer(x, y^2)),
            length(x))
image(log(z), main = "Autocorrelated Data")

hist(z)
ppt(z, w, main = "Null Distribution of Moran's I", xlab = "I")

# Geração de de um conjunto de dados sob hipótese nula.
z <- matrix(rnorm(length(x) * length(y), 0, 1/2), length(x))
image(z, main = "Uncorrelated Data")

hist(z)
ppt(z, w, main = "Null Distribution of Moran's I", xlab = "I")

#-----------------------------------------------------------------------
